//+------------------------------------------------------------------+
//|                                                   Swing line.mq4 |
//|                                                                  |
//| September 2010 issue                                             |
//| Technical Analysis of Stocks and Commodities                     |
//| Ron Black's article Getting Clear With Short-Term Swings       |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  1
#property indicator_color1   PaleVioletRed
#property indicator_width1   2

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT;
extern bool            Invert          = false;
extern string          ForSymbol       = "";
extern bool            alertsOn        = False;
extern bool            alertsOnCurrent = true;
extern bool            alertsMessage   = true;
extern bool            alertsSound     = false;
extern bool            alertsEmail     = false;

//
//
//
//
//

double trend[];
double line[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(2);
   SetIndexBuffer(0,line);
   SetIndexBuffer(1,trend);

   //
   //
   //
   //
   //
   
      if (ForSymbol=="") ForSymbol=Symbol();
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);

   //
   //
   //
   //
   //

   IndicatorShortName(ForSymbol+" "+timeFrameToString(TimeFrame)+" Swing line");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double  work[][4];
#define hHi 0
#define hLo 1
#define lHi 2
#define lLo 3

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { line[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //

   if (TimeFrame == Period() && ForSymbol==Symbol())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      for(i=limit,r=Bars-limit-1; i>=0; i--,r++)
      {
         int multiplier = 1; if (Invert) multiplier=-1;
         if (i==Bars-1)
         {
            work[r][hHi] = High[i]; work[r][hLo] = Low[i];
            work[r][lHi] = High[i]; work[r][lLo] = Low[i];
            trend[i]     = -1;
            line[i]      = trend[i]*multiplier;
            continue; 
         }
      
         //
         //
         //
         //
         //
      
         trend[i]     = trend[i+1];
         work[r][hHi] = work[r-1][hHi]; work[r][hLo] = work[r-1][hLo];
         work[r][lHi] = work[r-1][lHi]; work[r][lLo] = work[r-1][lLo];
 
               if (trend[i+1] == 1)
               {
                  work[r][hHi] = MathMax(work[r-1][hHi],High[i]);
                  work[r][hLo] = MathMax(work[r-1][hLo],Low[i]);
                     if (High[i]<work[r][hLo]) { trend[i] = -1; work[r][lHi] = High[i]; work[r][lLo] = Low[i]; }
               }
               if (trend[i+1] == -1)
               {
                  work[r][lHi] = MathMin(work[r-1][lHi],High[i]);
                  work[r][lLo] = MathMin(work[r-1][lLo],Low[i]);
                     if (Low[i]>work[r][lHi]) { trend[i] =  1; work[r][hHi] = High[i]; work[r][hLo] = Low[i]; }
               }
         line[i]      = trend[i]*multiplier;
      }
      manageAlerts();
   return(0);
   }            
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(ForSymbol,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(ForSymbol,TimeFrame,Time[i]);
         line[i] = iCustom(ForSymbol,TimeFrame,indicatorFileName,PERIOD_CURRENT,Invert,"",alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
   }
   manageAlerts();
   return(0);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"down");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  ForSymbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" "+timeFrameToString(_Period)+" Swing line changed to "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(ForSymbol+"Swing line ",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}